/*
 * Copyright 2002
 *  Bob Shaffer II
 *  4517 West Melissa Court
 *  Jacksonville, Florida 32210
 *  United States of America
 * All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 */

#include "FieldTypes.h"

#include <cstdlib>
#include <sstream>
#include <iomanip>
#include "textfiles.h"

/* -----------------------------------------------------------------
                     IPAddress Method Definitions
   ----------------------------------------------------------------- */

IPAddress::IPAddress(string ipstr) {
//    cerr<<"IPAddress::IPAddress: value=\'"<<ipstr<<"\'."<<endl;
    (*(string *)this)=ipstr;
    if ((*this)=="") (*this)=string( "0.0.0.0" );
//    cerr<<"IPAddress::IPAddress: value=\'"<<(*this)<<"\'."<<endl;
}

IPAddress::IPAddress(unsigned long ipval) {
    (*this)=longtoip(ipval);
}

unsigned long IPAddress::iptolong(string ipstr) {
    char *p=const_cast<char *>(ipstr.c_str());
    unsigned long val;
    val=atol(p)*16777216;
    while ((*p)!='.') ++p; ++p;
    val+=(atol(p)*65536);
    while ((*p)!='.') ++p; ++p;
    val+=(atol(p)*256);
    while ((*p)!='.') ++p; ++p;
    val+=atol(p);
    return val;
}

string IPAddress::longtoip(unsigned long ipval) {
    ostringstream temp;
    temp<<(ipval/16777216)<<'.'
        <<(ipval%16777216/65536)<<'.'
        <<(ipval%65536/256)<<'.'
        <<(ipval%256);
    return temp.str();
}

IPAddress::operator unsigned long() {
    return iptolong(*this);
}

//ostream &operator<<(ostream &out, IPAddress &myAddr) {
//    out<<(*this);
//    return out;
//}

//unsigned long IPAddress::operator=(unsigned long ipval) {
//    (string)(*this)=longtoip(ipval);
//    return ipval;
//}

/* -----------------------------------------------------------------
                     LongField Method Definitions
   ----------------------------------------------------------------- */

LongField::LongField(string LongSource) {
    Value = LongSource;
}

LongField::LongField(long LongSource) {
    ostringstream temp;
    temp<<LongSource;
    Value = temp.str();
}

LongField::operator long() {
    return atol(Value.c_str());
}

LongField::operator string() {
    return Value;
}

LongField LongField::operator ++(int cute) {
    LongField retval=(*this);
    (*this)=long(*this)+1;
    return retval;
}

LongField LongField::operator --(int cute) {
    LongField retval=(*this);
    (*this)=long(*this)-1;
    return retval;
}

LongField LongField::operator +=(long rval) {
    (*this)=long(*this)+rval;
    return (*this);
}

LongField LongField::operator -=(long rval) {
    (*this)=long(*this)-rval;
    return (*this);
}

LongField LongField::operator ++() {
    return (*this)+=1;
}

LongField LongField::operator --() {
    return (*this)-=1;
}

/*
 * class DateTimeField {
 *     friend ostream;
 *     public:
 *     DateTimeField(string DateTimeSource = "0000-0-0 00:00:00");
 *     DateTimeField(time_t TimeTSource);
 * //    operator long();
 *     operator time_t();
 *     operator struct tm();
 *     operator string();
 *     private:
 *     string Value;
 * } DateTimeField_Now(time(NULL));
*/
DateTimeField::DateTimeField(string DateTimeSource) {
    Value=DateTimeSource;
}

DateTimeField::DateTimeField(time_t TimeTSource) {
    struct tm temp=*(localtime(&TimeTSource));
    ostringstream ValueStream;
    ValueStream<<(temp.tm_year+1900)<<'-'
               <<(temp.tm_mon+1)<<'-'
               <<(temp.tm_mday)<<' '
               <<setfill('0')<<setw(2)<<(temp.tm_hour)<<':'
               <<setfill('0')<<setw(2)<<(temp.tm_min)<<':'
               <<setfill('0')<<setw(2)<<(temp.tm_sec);
    Value=ValueStream.str();
}

DateTimeField::DateTimeField(struct tm TmSource) {
    ostringstream ValueStream;
    ValueStream<<(TmSource.tm_year+1900)<<'-'
               <<(TmSource.tm_mon+1)<<'-'
               <<(TmSource.tm_mday)<<' '
               <<setfill('0')<<setw(2)<<(TmSource.tm_hour)<<':'
               <<setfill('0')<<setw(2)<<(TmSource.tm_min)<<':'
               <<setfill('0')<<setw(2)<<(TmSource.tm_sec);
    Value=ValueStream.str();
}

DateTimeField::operator time_t() {
    struct tm temp;
    temp=(*this);
    return mktime(&temp);
}

DateTimeField::operator struct tm() {
    Delimited_Row split_DateTime, split_Date, split_Time;
    split_DateTime.set_delimiter(" ");
    split_DateTime.set_row(Value);
    split_Date.set_delimiter("-");
    split_Date.set_row(split_DateTime.cell(0));
    split_Time.set_delimiter(":");
    split_Time.set_row(split_DateTime.cell(1));
    struct tm retval;
    retval.tm_year=atoi(split_Date.cell(0).c_str())-1900;
    retval.tm_mon=atoi(split_Date.cell(1).c_str())-1;
    retval.tm_mday=atoi(split_Date.cell(2).c_str());
    retval.tm_hour=atoi(split_Time.cell(0).c_str());
    retval.tm_min=atoi(split_Time.cell(1).c_str());
    retval.tm_sec=atoi(split_Time.cell(2).c_str());
    // This call sets wday, yday, and isdst accordingly
    mktime(&retval);
    return retval;
}

DateTimeField::operator string() {
    return Value;
}

int DateTimeField::Year() {
    Delimited_Row split_DateTime, split_Date;
    split_DateTime.set_delimiter(" ");
    split_DateTime.set_row(Value);
    split_Date.set_delimiter("-");
    split_Date.set_row(split_DateTime.cell(0));
    return atoi(split_Date.cell(0).c_str());
}

int DateTimeField::Month() {
    Delimited_Row split_DateTime, split_Date;
    split_DateTime.set_delimiter(" ");
    split_DateTime.set_row(Value);
    split_Date.set_delimiter("-");
    split_Date.set_row(split_DateTime.cell(0));
    return atoi(split_Date.cell(1).c_str());
}

int DateTimeField::Day() {
    Delimited_Row split_DateTime, split_Date;
    split_DateTime.set_delimiter(" ");
    split_DateTime.set_row(Value);
    split_Date.set_delimiter("-");
    split_Date.set_row(split_DateTime.cell(0));
    return atoi(split_Date.cell(2).c_str());
}

int DateTimeField::Hour() {
    Delimited_Row split_DateTime, split_Time;
    split_DateTime.set_delimiter(" ");
    split_DateTime.set_row(Value);
    split_Time.set_delimiter(":");
    split_Time.set_row(split_DateTime.cell(1));
    return atoi(split_Time.cell(0).c_str());
}

int DateTimeField::Minute() {
    Delimited_Row split_DateTime, split_Time;
    split_DateTime.set_delimiter(" ");
    split_DateTime.set_row(Value);
    split_Time.set_delimiter(":");
    split_Time.set_row(split_DateTime.cell(1));
    return atoi(split_Time.cell(1).c_str());
}

int DateTimeField::Second() {
    Delimited_Row split_DateTime, split_Time;
    split_DateTime.set_delimiter(" ");
    split_DateTime.set_row(Value);
    split_Time.set_delimiter(":");
    split_Time.set_row(split_DateTime.cell(1));
    return atoi(split_Time.cell(2).c_str());
}

int DateTimeField::DOW() {
    return (to_days()-1)%7;
}

unsigned long DateTimeField::to_days() {
    static int daysToMonth[] = { 0,31,59,90,120,151,181,212,243,273,304,334 };
    int year( Year() ),
        month( Month() ),
        day( Day() );
    return ((year*146097-97)/400+daysToMonth[month-1]+(month>3&&year%4==0&&(year%400==0||year%100!=0))+day);
}

void DateTimeField::from_days(unsigned long days) {
    int year( (days*400+97)/146097 ), ydays( days-(year*146097-97)/400 ), month( 12 ), day, leap( year%4==0&&(year%400==0||year%100!=0) );
    int daysToMonth[] = { 0,31,59+leap,90+leap,120+leap,151+leap,181+leap,212+leap,243+leap,273+leap,304+leap,334+leap };
    while (daysToMonth[month-1]>=ydays) --month;
    day=ydays-daysToMonth[month-1];
    ostringstream valueBuf;
    valueBuf<<setw(4)<<setfill('0')<<year<<'-'<<setw(2)<<setfill('0')<<month<<'-'<<setw(2)<<setfill('0')<<day<<" 00:00:00";
    Value=valueBuf.str();
}

unsigned long long DateTimeField::to_seconds() {
    unsigned long long day_seconds( +Hour()*3600+Minute()*60+Second() );
    return ((unsigned long long)to_days()*86400+day_seconds);
}

void DateTimeField::from_seconds(unsigned long long seconds) {
    unsigned long days( seconds/86400 );
    int year( (days*400+97)/146097 ), ydays( days-(year*146097-97)/400 ), month( 12 ), day, leap( year%4==0&&(year%400==0||year%100!=0) );
    int hour( (seconds%86400)/3600 ), minute( (seconds%3600)/60 ), second( seconds%60 );
    int daysToMonth[] = { 0,31,59+leap,90+leap,120+leap,151+leap,181+leap,212+leap,243+leap,273+leap,304+leap,334+leap };
    while (daysToMonth[month-1]>=ydays) --month;
    day=ydays-daysToMonth[month-1];
    ostringstream valueBuf;
    valueBuf<<setw(4)<<setfill('0')<<year<<'-'<<setw(2)<<setfill('0')<<month<<'-'<<setw(2)<<setfill('0')<<day<<' '
	    <<setw(2)<<setfill('0')<<hour<<':'<<setw(2)<<setfill('0')<<minute<<':'<<setw(2)<<setfill('0')<<second;
    Value=valueBuf.str();
}

ostream &operator<<(ostream &out, DateTimeField &myObj) {
    out<<(string)myObj;
    return out;
}

DateField::DateField(string DateSource) {
    Value=DateSource;
}

DateField::DateField(time_t TimeTSource) {
    struct tm temp=*(localtime(&TimeTSource));
    ostringstream ValueStream;
    ValueStream<<(temp.tm_year+1900)<<'-'
               <<(temp.tm_mon+1)<<'-'
               <<(temp.tm_mday);
    Value=ValueStream.str();
}

DateField::operator time_t() {
    struct tm temp;
    temp=(*this);
    return mktime(&temp);
}

DateField::operator struct tm() {
    Delimited_Row split_Date;
    split_Date.set_delimiter("-");
    split_Date.set_row(Value);
    struct tm retval;
    retval.tm_year=atoi(split_Date.cell(0).c_str())-1900;
    retval.tm_mon=atoi(split_Date.cell(1).c_str())-1;
    retval.tm_mday=atoi(split_Date.cell(2).c_str());
    retval.tm_hour=0;
    retval.tm_min=0;
    retval.tm_sec=0;
    // This call sets wday, yday, and isdst accordingly
    mktime(&retval);
    return retval;
}

DateField::operator string() {
    return Value;
}

int DateField::Year() {
    Delimited_Row split_Date;
    split_Date.set_delimiter("-");
    split_Date.set_row(Value);
    return atoi(split_Date.cell(0).c_str());
}

int DateField::Month() {
    Delimited_Row split_Date;
    split_Date.set_delimiter("-");
    split_Date.set_row(Value);
    return atoi(split_Date.cell(1).c_str());
}

int DateField::Day() {
    Delimited_Row split_Date;
    split_Date.set_delimiter("-");
    split_Date.set_row(Value);
    return atoi(split_Date.cell(2).c_str());
}

int DateField::DOW() {
    return (to_days()-1)%7;
}

unsigned long DateField::to_days() {
    static int daysToMonth[] = { 0,31,59,90,120,151,181,212,243,273,304,334 };
    int year( Year() ),
        month( Month() ),
        day( Day() );
    return ((year*146097-97)/400+daysToMonth[month-1]+(month>3&&year%4==0&&(year%400==0||year%100!=0))+day);
}

void DateField::from_days(unsigned long days) {
    int year( (days*400+97)/146097 ), ydays( days-(year*146097-97)/400 ), month( 12 ), day, leap( year%4==0&&(year%400==0||year%100!=0) );
    int daysToMonth[] = { 0,31,59+leap,90+leap,120+leap,151+leap,181+leap,212+leap,243+leap,273+leap,304+leap,334+leap };
    while (daysToMonth[month-1]>=ydays) --month;
    day=ydays-daysToMonth[month-1];
    ostringstream valueBuf;
    valueBuf<<setw(4)<<setfill('0')<<year<<'-'<<setw(2)<<setfill('0')<<month<<'-'<<setw(2)<<setfill('0')<<day;
    Value=valueBuf.str();
}

ostream &operator<<(ostream &out, DateField &myObj) {
    out<<(string)myObj;
    return out;
}

TimeField::TimeField(string TimeSource) {
    Value=TimeSource;
}

TimeField::TimeField(time_t TimeTSource) {
    struct tm temp=*(localtime(&TimeTSource));
    ostringstream ValueStream;
    ValueStream<<setfill('0')<<setw(2)<<(temp.tm_hour)<<':'
               <<setfill('0')<<setw(2)<<(temp.tm_min)<<':'
               <<setfill('0')<<setw(2)<<(temp.tm_sec);
    Value=ValueStream.str();
}

TimeField::operator time_t() {
    struct tm temp;
    temp=(*this);
    return mktime(&temp);
}

TimeField::operator struct tm() {
    Delimited_Row split_Time;
    split_Time.set_delimiter(":");
    split_Time.set_row(Value);
    struct tm retval;
    retval.tm_year=0;
    retval.tm_mon=1;
    retval.tm_mday=1;
    retval.tm_hour=atoi(split_Time.cell(0).c_str());
    retval.tm_min=atoi(split_Time.cell(1).c_str());
    retval.tm_sec=atoi(split_Time.cell(2).c_str());
    // This call sets wday, yday, and isdst accordingly
    mktime(&retval);
    return retval;
}

TimeField::operator string() {
    return Value;
}

int TimeField::Hour() {
    Delimited_Row split_Time;
    split_Time.set_delimiter(":");
    split_Time.set_row(Value);
    return atoi(split_Time.cell(0).c_str());
}

int TimeField::Minute() {
    Delimited_Row split_Time;
    split_Time.set_delimiter(":");
    split_Time.set_row(Value);
    return atoi(split_Time.cell(1).c_str());
}

int TimeField::Second() {
    Delimited_Row split_Time;
    split_Time.set_delimiter(":");
    split_Time.set_row(Value);
    return atoi(split_Time.cell(2).c_str());
}

ostream &operator<<(ostream &out, TimeField &myObj) {
    out<<(string)myObj;
    return out;
}

FlagField::FlagField(int initVal) {
    Value(initVal);
}

FlagField::FlagField(string initVal) {
    Value(atoi(initVal.c_str()));
}

FlagField::FlagField(const char *initVal) {
    Value(atoi(initVal));
}

int FlagField::Value() {
    int retval=0;
    for (int bitn=0; bitn<32; ++bitn) {
        retval |= (flags[bitn] << bitn);
    }
    return retval;
}

void FlagField::Value(int newVal) {
    for (int bitn=0; bitn<32; ++bitn) {
        flags[bitn] = newVal & ( 0x00000001 << bitn );
    }
}

bool &FlagField::flag(int num) {
    static bool outofrangebool = false;
    if (num<0 || num>31) return outofrangebool;
    return flags[num];
}

string &FlagField::name(int num) {
    static string outofrangestring = "Out_Of_Range";
    if (num<0 || num>31) return outofrangestring;
    return names[num];
}

string FlagField::str() {
    ostringstream temp;
    temp<<Value();
    return temp.str();
}

bool &FlagField::operator[](int num) {
    return flag(num);
}

bool &FlagField::operator[](const char *flagName) {
    if (flagName==NULL) return flag(-1);
    for (int bitn=0; bitn<32; ++bitn) {
        if (names[bitn]==flagName) return flags[bitn];
    }
    return flag(-1);
}

bool &FlagField::operator[](string flagName) {
    for (int bitn=0; bitn<32; ++bitn) {
        if (names[bitn]==flagName) return flags[bitn];
    }
    return flag(-1);
}

FlagField FlagField::operator&(FlagField other) {
    return FlagField(Value() & other.Value());
}

FlagField FlagField::operator&(int other) {
    return FlagField(Value() & other);
}

FlagField FlagField::operator|(FlagField other) {
    return FlagField(Value() | other.Value());
}

FlagField FlagField::operator|(int other) {
    return FlagField(Value() | other);
}


