/*
 * Copyright 2002
 *  Bob Shaffer II
 *  4517 West Melissa Court
 *  Jacksonville, Florida 32210
 *  United States of America
 * All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 */

#ifndef FIELDTYPES_H_INCLUDED
#define FIELDTYPES_H_INCLUDED 1

#include <iostream>
#include <string>
#include <ctime>

using namespace std;

class IPAddress : public string {
    public:
    IPAddress(string ipstr);
    IPAddress(unsigned long ipval=0);
    unsigned long iptolong(string ipstr);
    string longtoip(unsigned long ipval);
    operator unsigned long();
//    unsigned long operator=(unsigned long ipval);
//    string address
};

//ostream &operator<<(ostream &out, IPAddress &myAddr);

class LongField {
    friend class std::ostream;
    public:
    LongField(string LongSource = "0");
    LongField(long LongSource);
    operator long();
    operator string();
    LongField operator ++(int cute);
    LongField operator --(int cute);
    LongField operator +=(long rval);
    LongField operator -=(long rval);
    LongField operator ++();
    LongField operator --();
    private:
    string Value;
};

class DateField {
    friend class std::ostream;
    public:
    DateField(string DateTimeSource = "0000-0-0");
    DateField(time_t TimeTSource);
    operator time_t();
    operator struct tm();
    operator string();
    int Year();
    int Month();
    int Day();
    unsigned long to_days();
    void from_days(unsigned long days);
    int DOW();
    private:
    string Value;
} ;

ostream &operator<<(ostream &out, DateField &myObj);

class TimeField {
    friend class std::ostream;
    public:
    TimeField(string DateTimeSource = "00:00:00");
    TimeField(time_t TimeTSource);
    operator time_t();
    operator struct tm();
    operator string();
    int Hour();
    int Minute();
    int Second();
    private:
    string Value;
} ;

ostream &operator<<(ostream &out, TimeField &myObj);

class DateTimeField {
    friend class std::ostream;
    public:
    DateTimeField(string DateTimeSource = "0000-0-0 00:00:00");
    DateTimeField(time_t TimeTSource);
    DateTimeField(struct tm TmSource);
    operator time_t();
    operator struct tm();
    operator string();
    int Year();
    int Month();
    int Day();
    int Hour();
    int Minute();
    int Second();
    unsigned long to_days();
    void from_days(unsigned long days);
    unsigned long long to_seconds();
    void from_seconds(unsigned long long seconds);
    int DOW();
    private:
    string Value;
} ;

ostream &operator<<(ostream &out, DateTimeField &myObj);

class FlagField {
    public:
    FlagField(int initVal=0);
    FlagField(string initVal);
    FlagField(const char *initVal);
    virtual int Value();
    virtual void Value(int newVal);
    virtual bool &flag(int num);
    virtual string &name(int num);
    virtual string str();
    virtual bool &operator[](int num);
    virtual bool &operator[](const char *flagName);
    virtual bool &operator[](string flagName);
    FlagField operator&(FlagField other);
    FlagField operator&(int other);
    FlagField operator|(FlagField other);
    FlagField operator|(int other);
    private:
    bool flags[32];
    string names[32];
};

#endif

