/*
 * Copyright 2002
 *  Bob Shaffer II
 *  4517 West Melissa Court
 *  Jacksonville, Florida 32210
 *  United States of America
 * All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 */

#ifndef MYSQL_DATABASE_INCLUDED
#define MYSQL_DATABASE_INCLUDED 1

#include "textfiles.h"
#include <mysql/mysql.h>

using namespace std;

/*
    The MySQL_Table class requires an auto_increment ID key field to work.
    It must be created by the friend class MySQL_Database.
    The table is completely updated accordingly when a destructor is called.
    You may call 'Update' as often as you like without unnecessary updates being made.
*/

class MySQL_Database;

/*
    class MySQL_Table Notes:
    The properties of 'Table' should be accessable by members of this
    class as well as the friend class MSQL_Database

    I'm going with the concept of "letting the compiler do the work".  If string &cell
    is called, we can assume that the user has chosen an operation which updates the
    data since it was needed as an lvalue.

*/

class MySQL_Table {
    friend class MySQL_Database;
    public:
    MySQL_Table();
    MySQL_Table(MySQL_Database &DBRef,
                const char *table,
                const char *fields="*",
                const char *params=NULL);
    ~MySQL_Table();
    string &cell(long row=0, long col=1);
    string &cell(long row=0, string cellName="ID");
    Table &GetTable();
    long FieldIndex(string FieldName);
    long ID_Index();
    my_ulonglong ID(long row=0);
    long length();
    long Insert(Delimited_Row &NewRow);
    void Remove(long row=0);
    void Update();
    private:
    Table Data;
    Table Original;
    MySQL_Database *mDB;
    Comma_Delimited_Row Fields;
    string TableName;
};

class MySQL_Database {
    friend class MySQL_Table;
    public:
    MySQL_Database(const char *database,
                   const char *user,
                   const char *password=NULL,
                   const char *host="localhost",
                   int port=0);
    ~MySQL_Database();
    void Direct_Query(Table &Result, string query, Delimited_Row *FieldList=NULL);
    void Direct_Query(string query);
    void Query(MySQL_Table &Result,
               const char *table,
               const char *fields="*",
               const char *params=NULL);
    int errornum(); // 0 if no error
    string Escape_String(string s);
    private:
    MYSQL DB;
};

#endif

