/*
 * Copyright 2002
 *  Bob Shaffer II
 *  4517 West Melissa Court
 *  Jacksonville, Florida 32210
 *  United States of America
 * All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 */
#include "mailfiles.h"
#include <cstring>
#include <cstdlib>
#include <unistd.h>

/*
typedef struct {
    streampos where;
    string from;
    string subject;
    string date;
} mailfile_index_entry;

class mailfile_index : public vector<mailfile_index_entry> {
    friend istream;
    friend ostream;
};
istream &operator>>(istream &in, mailfile_index &myObj);
ostream &operator<<(ostream &out, mailfile_index &myObj);
*/

istream &operator>>(istream &in, mailfile_index &myObj) {
    mailfile_index_entry curEnt;
    char Buf[5000], *p;
    streampos BufPos;
    int Filled_In;
    bool FoundMsg( false );
    BufPos=in.tellg();
    while (in.getline(Buf, 5000, '\n')) {

        if (strncmp(Buf, "From ", 5)==0) {
            FoundMsg=true;
            if (BufPos>0) {
                curEnt.length=BufPos-curEnt.where;
                myObj.push_back(curEnt);
            }
            curEnt.where=BufPos;
            curEnt.from=curEnt.date=curEnt.subject="";
            Filled_In=0;
            do {
                if (!in.getline(Buf, 5000, '\n')) break;
                if (strncmp(Buf, "From: ", 6)==0) {
                    for (p=Buf+6; isspace(*p); ++p);
                    curEnt.from=p;
                    ++Filled_In;
                } else if (strncmp(Buf, "Subject: ", 9)==0) {
                    for (p=Buf+9; isspace(*p); ++p);
                    curEnt.subject=p;
                    ++Filled_In;
                } else if (strncmp(Buf, "Date: ", 6)==0) {
                    for (p=Buf+6; isspace(*p); ++p);
                    curEnt.date=p;
                    ++Filled_In;
                }
            } while (Buf[0]!='\0' && Filled_In<3);
        }
        BufPos=in.tellg();
    }
    curEnt.length=BufPos-curEnt.where;
    if (FoundMsg) myObj.push_back(curEnt);
    return in;
}

ostream &operator<<(ostream &out, mailfile_index &myObj) {
    for (long n=0; n<myObj.size(); ++n)
        out<<myObj[n].where<<'\t'<<myObj[n].length<<'\t'<<myObj[n].date
           <<'\t'<<myObj[n].from<<'\t'<<myObj[n].subject<<endl;
    return out;
}

string message::From() {
    for (long n=1; n<Header.size(); ++n) {
        if (string(Header[n], 0, 6)==string("From: ")) return string(Header[n], 6);
    }
    return string("(Sender Not Specified)");
}

string message::Subject() {
    for (long n=1; n<Header.size(); ++n) {
        if (string(Header[n], 0, 9)==string("Subject: ")) return string(Header[n], 9);
    }
    return string("(No Subject)");
}

string message::Date() {
    for (long n=1; n<Header.size(); ++n) {
        if (string(Header[n], 0, 6)==string("Date: ")) return string(Header[n], 6);
    }
    return string("(Date Not Found)");
}

string message::ContentType() {
    for (long n=1; n<Header.size(); ++n) {
        if (Header[n].substr(0, 14)==string("Content-Type: ")) {
            long endCT=Header[n].find(';', 14);
            return string(Header[n], 14, endCT-14);
        }
    }
    return string("TEXT/PLAIN"); // just use plain text for default
}

void message::clear() {
    Header.clear();
    Text.clear();
}

istream &operator>>(istream &in, message &myobj) {
    myobj.clear();
    string Buf;
    streampos PreBufPos;
    bool GotHeader=false;
    for (;;) {
        PreBufPos=in.tellg();
        getline(in, Buf);
        if (in.eof()) break;
        if (GotHeader) {
            if (Buf.substr(0, 5)==string("From ")) {
                in.seekg(PreBufPos, ios::beg);
                break;
            } else {
                myobj.Text.push_back(Buf);
            }
        } else {
            if (Buf.length()==0) {
                GotHeader=true;
            } else {
                myobj.Header.push_back(Buf);
            } // end "if (Buf.length...else"
        } // end "if (GotHeader...else"
    } // end "for (;;)"
    return in;
}

ostream &operator<<(ostream &out, message &myobj) {
    for (long n=0; n<myobj.Header.size(); ++n) {
        out<<myobj.Header[n]<<endl;
    }
    out<<endl;
    for (long n=0; n<myobj.Text.size(); ++n) {
        out<<myobj.Text[n]<<endl;
    }
    out<<endl;
    return out;
}

/*    class mailfile's private members:
      ---------------------------------
        iostream *MailFileStreamPtr;
          mailfile_index MsgIndex;
               long Changed; // lowest record changed
*/

mailfile::mailfile(const char *mfName) {
    FileName=mfName;
    ifstream MsgFile(mfName);
    MsgFile>>MsgIndex;
    Changed=MsgIndex.size();
}

/*
mailfile::mailfile(iostream &mfStream) {
    MailFileStreamPtr=&mfStream;
    ((istream &)(*MailFileStreamPtr)) >> MsgIndex;
    Changed=MsgIndex.size();
}
*/

long mailfile::messages() {
    return long(MsgIndex.size());
}

void mailfile::remove(long msgNum) {
    if (Changed>msgNum) Changed=msgNum;
    MsgIndex.erase(MsgIndex.begin()+msgNum);
}

void mailfile::save() {
    if (Changed<MsgIndex.size()) {
        ifstream in(FileName.c_str());
        vector<char *> msgBuf(MsgIndex.size());
        for (long m=0; m<MsgIndex.size(); ++m) {
            msgBuf[m]=new char[MsgIndex[m].length+1];
            if (msgBuf[m]==NULL) {
                cerr<<"Out of memory.\n";
                exit(-1);
            }
            in.seekg(MsgIndex[m].where);
            in.read(msgBuf[m], MsgIndex[m].length);
            msgBuf[m][MsgIndex[m].length]='\0';
        }
        in.close();
        // Now I have an array of raw messages to append after message [Changed-1]
        // ostream *oP=MailFileStreamPtr;
        fstream out(FileName.c_str(), ios::out);
        // Now let's dump the messages
        for (long m=0; m<msgBuf.size(); ++m) {
            out<<(msgBuf[m]);
            delete [] (msgBuf[m]);
        }
        out.close();
    }
}

void mailfile::get(long msgNum, message &M) {
    if (MsgIndex.size()<=msgNum) return;
    ifstream in(FileName.c_str());
    in.seekg(MsgIndex[msgNum].where);
    in>>M;
}

void mailfile::append(message &M) {
    ofstream out(FileName.c_str(), ios::app);
    mailfile_index_entry E;
    E.where=out.tellp();
    out<<M;
    E.length=out.tellp()-E.where;
    E.from=M.From();
    E.date=M.Date();
    E.subject=M.Subject();
    MsgIndex.push_back(E);
    if (Changed==MsgIndex.size()-1) ++Changed;
}

/*
void mailfile::save() {
    if (Changed<MsgIndex.size()) {
        iostream *ioP=MailFileStreamPtr;
        vector<char *> msgBuf(MsgIndex.size()-Changed);
        cerr<<"Reading "<<msgBuf.size()<<" messages...\n";
        for (long m=Changed; m<MsgIndex.size(); ++m) {
            msgBuf[m-Changed]=new char[MsgIndex[m].length+1];
            if (msgBuf[m-Changed]==NULL) {
                cerr<<"Out of memory.\n";
                exit(-1);
            }
            (*ioP).seekp(MsgIndex[m].where);
            (*ioP).read(msgBuf[m-Changed], MsgIndex[m].length);
            msgBuf[m-Changed][MsgIndex[m].length]='\0';
        }
        // Now I have an array of raw messages to append after message [Changed-1]
        // ostream *oP=MailFileStreamPtr;
        if (Changed>0) {
            (*ioP).seekp(MsgIndex[Changed-1].where+MsgIndex[Changed-1].length);
        } else (*ioP).seekp(0);
        // Now let's dump the messages
        cerr<<"Writing...\n";
        for (long m=0; m<msgBuf.size(); ++m) {
            cerr<<(msgBuf[m]);
            (*ioP)<<(msgBuf[m]);
            delete [] (msgBuf[m]);
        }
        (*ioP)<<ends; // append EOF
    }
}

void mailfile::get(long msgNum, message &M) {
    istream *iP=MailFileStreamPtr;
    (*iP).seekg(MsgIndex[msgNum].where);
    (*iP)>>M;
}

void mailfile::append(message &M) {
    ostream *oP=MailFileStreamPtr;
    (*oP).seekp(0, ios::end);
    mailfile_index_entry E;
    E.where=(*oP).tellp();
    (*oP)<<M;
    E.length=(*oP).tellp()-E.where;
    E.from=M.From();
    E.date=M.Date();
    E.subject=M.Subject();
    MsgIndex.push_back(E);
    if (Changed==MsgIndex.size()-1) ++Changed;
}

*/

void mailfile::output_index(ostream &out) {
    out<<MsgIndex;
}
