/*
 * Copyright 2002
 *  Bob Shaffer II
 *  4517 West Melissa Court
 *  Jacksonville, Florida 32210
 *  United States of America
 * All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 */

#ifdef MAILFILES_INCLUDED
#define MAILFILES_INCLUDED 1

#include <iostream>
#include <fstream>
#include <string>
#include <vector>

using namespace std;

typedef struct {
    streampos where;
    long length;
    string from;
    string subject;
    string date;
} mailfile_index_entry;

class mailfile_index : public vector<mailfile_index_entry> {
/*
    This class is weird.  It takes a mail-file as input ( >> )
    and produces a tab-delimited index with fields
    { offset, date, from, subject } for output ( << ).
*/
    friend class std::istream;
    friend class std::ostream;
};

istream &operator>>(istream &in, mailfile_index &myObj);
ostream &operator<<(ostream &out, mailfile_index &myObj);

class message {
    friend class std::istream;
    friend class std::ostream;
    public:
    vector<string> Header;
    vector<string> Text;
    virtual string From();
    virtual string Subject();
    virtual string Date();
    virtual string ContentType();
    virtual void clear();
};

istream &operator>>(istream &in, message &myobj);
ostream &operator<<(ostream &out, message &myobj);

class mailfile {
    public:
    //mailfile(iostream &mfStream);
    mailfile(const char *mfName);
    long messages();
    void remove(long msgNum);
    void save();
    void get(long msgNum, message &M);
    void append(message &M);
    void output_index(ostream &out);
    private:
//    iostream *MailFileStreamPtr;
    string FileName;
    mailfile_index MsgIndex;
    long Changed;
};
#endif

