/*
 * Copyright 2002
 *  Bob Shaffer II
 *  4517 West Melissa Court
 *  Jacksonville, Florida 32210
 *  United States of America
 * All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 */

#include "textfiles.h"
#include <string.h>
#include <algorithm>

void Delimited_Row::set_delimiter(const char *newdel) {
    delimiter=newdel;
}

const char *Delimited_Row::get_delimiter() {
    return delimiter.c_str();
}

string & Delimited_Row::cell(long column) {
    while (column>=size()) push_back(string(""));
    return (*this)[column];
}

void Delimited_Row::set_row(string newrow) {
    parse_in(newrow);
}

string Delimited_Row::get_row() {
    return parse_out();
}

void Delimited_Row::parse_in(string input) {
    const char *p=input.c_str(), *d=delimiter.c_str();
    long col=0;
    if (size()>0) clear();
    push_back(string("\0"));
    while (*p != '\0') {
        if (strncmp(p,d,delimiter.length())==0) {
            p+=delimiter.length();
            (*this)[col]+="\0";
            ++col;
            push_back(string("\0"));
        } else {
            (*this)[col]+=*p;
            ++p;
        }
    }
}

string Delimited_Row::parse_out() {
    string retval;
    if (size()>0) retval=(*this)[0];
    else return string("\0");
    for (long col=1; col<size(); ++col) {
        retval+=delimiter+(*this)[col];
    }
    return retval;
}

Delimited_Row & Delimited_Row::operator=(string Other) {
    parse_in(Other);
    return (*this);
}

istream &operator>>(istream &in, Delimited_Row &MyRow) {
    string temp;
    getline(in, temp);
    MyRow.set_row(temp);
    return in;
}

ostream &operator<<(ostream &out, Delimited_Row &MyRow) {
    out<<MyRow.get_row()<<endl;
    return out;
}

Table::Table(const char *Delim) {
    delimiter=Delim;
}

long Table::find(const char *what, long row, long column) {
    string What(what);
    iterator E=begin()+row;
    long retval=row;
    while (E!=end()) {
        if ((*E)[column]==What) return retval;
        ++E;
        ++retval;
    }
    return -1;
}

long Table::scan(const char *what, long row, long column) {
    string What(what);
    iterator E=begin()+row;
    long retval=row;
    while (E!=end()) {
        if (isSubStr((*E)[column],What)) return retval;
        ++E;
        ++retval;
    }
    return -1;
}

long Table::ifind(const char *what, long row, long column) {
    string What(strToLower(string(what)));
    iterator E=begin()+row;
    long retval=row;
    while (E!=end()) {
        if (strToLower((*E)[column])==What) return retval;
        ++E;
        ++retval;
    }
    return -1;
}

long Table::iscan(const char *what, long row, long column) {
    string What(strToLower(string(what)));
    iterator E=begin()+row;
    long retval=row;
    while (E!=end()) {
        if (isSubStr(strToLower((*E)[column]),What)) return retval;
        ++E;
        ++retval;
    }
    return -1;
}

Delimited_Row & Table::getrow(long row) {
    while (row>=size()) addrow(string(""));
    return (*this)[row];
}

long Table::addrow(string newrow) {
    Delimited_Row temp;
    temp.set_delimiter(delimiter.c_str());
    temp.set_row(newrow);
    push_back(temp);
    return length()-1;
}

void Table::remrow(long row) {
    erase(begin()+row);
}

void Table::set_delimiter(const char *newdel) {
    delimiter=newdel;
}

const char * Table::get_delimiter() {
    return delimiter.c_str();
}

string & Table::cell(long row, long column) {
    return getrow(row).cell(column);
}

long Table::length() {
    return size();
}

void Table::Sort(long column, int keytype) {
    sort(begin(), end(), Tab_Sort_Comp(column, keytype));
}

string Table::strToLower(string s) {
    string retval;
    for (int i=0; i<s.length(); ++i) retval+=tolower(s[i]);
    return retval;
}

bool Table::isSubStr(string s1, string s2) {
    return (strstr(s1.c_str(), s2.c_str())!=NULL);
}

istream &operator>>(istream &in, Table &MyTab) {
    string S;
    for (;;) {
        getline(in,S);
        if (in.eof()) break;
        MyTab.addrow(S);
    }
    return in;
}

ostream &operator<<(ostream &out, Table &MyTab) {
    Table::iterator E=MyTab.begin();
    while (E!=MyTab.end()) {
        (*E).set_delimiter(MyTab.get_delimiter());
        out<<*E;
        ++E;
    }
    return out;
}

Tab_Sort_Comp::Tab_Sort_Comp(long column, int keytype) {
    SortCol=column;
    SortKey=keytype;
}

int Tab_Sort_Comp::operator()(const Delimited_Row &left, const Delimited_Row &right) {
    switch (SortKey) {
        case 0:
        case 1:  // String (Ascending)
            {
                return (left[SortCol]<right[SortCol]);
            }
            break;
        case -1: // String (Descending)
            {
                return (left[SortCol]>right[SortCol]);
            }
            break;
        case 2:  // Integer (Ascending)
            {
                return (atol(left[SortCol].c_str())<atol(right[SortCol].c_str()));
            }
            break;
        case -2: // Integer (Descending)
            {
                return (atol(left[SortCol].c_str())>atol(right[SortCol].c_str()));
            }
            break;
        case 3:  // Real (Ascending)
            {
                return (atof(left[SortCol].c_str())<atof(right[SortCol].c_str()));
            }
            break;
        case -3: // Real (Descending)
            {
                return (atof(left[SortCol].c_str())>atof(right[SortCol].c_str()));
            }
            break;
        // end of case statements
    } // end of switch
}
