/*
 * Copyright 2002
 *  Bob Shaffer II
 *  4517 West Melissa Court
 *  Jacksonville, Florida 32210
 *  United States of America
 * All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 */

#ifndef textfiles_h_included
#define textfiles_h_included 1
#include <string>
#include <vector>
#include <iostream>

using namespace std;

class Delimited_Row : public vector<string> {
     friend class std::istream;
     friend class std::ostream;
    public:
        virtual void set_delimiter(const char *newdel);
        virtual const char *get_delimiter();
//        virtual Delimited_Row & operator =  // should generate default //
//        virtual string & operator string();
        virtual string & cell(long column=0);
        virtual void set_row(string newrow);
        virtual string get_row();
        virtual Delimited_Row & operator=(string Other);
        virtual long length() { return size(); }
    private:
        void parse_in(string);
        string parse_out();
//        vector<string> row;
        string delimiter;
};

istream & operator >> (istream & in, Delimited_Row &MyRow);
ostream & operator << (ostream & out, Delimited_Row &MyRow);

class Tab_Delimited_Row : public Delimited_Row {
        public:
        Tab_Delimited_Row() { set_delimiter("\t"); }
};

class Comma_Delimited_Row : public Delimited_Row {
        public:
        Comma_Delimited_Row() { set_delimiter(","); }
};

class Table : public vector<Delimited_Row> {
    friend class std::istream;
    friend class std::ostream;
    public:
        Table(const char *Delim="");
        virtual long find(const char *what, long row=0, long column=0);
        virtual long scan(const char *what, long row=0, long column=0);
        virtual long ifind(const char *what, long row=0, long column=0);
        virtual long iscan(const char *what, long row=0, long column=0);
        virtual Delimited_Row & getrow(long row=0);
        virtual long addrow(string newrow);
        virtual void remrow(long row);
        virtual string & cell(long row=0, long column=0);
        virtual long length();
        virtual void set_delimiter(const char *newdel);
        virtual const char * get_delimiter();
        virtual void Sort(long column, int keytype);
//        virtual vector<Delimited_Row>::iterator table_iterator();
    private:
//        vector<Delimited_Row> rows;
        string delimiter;
        static string strToLower(string s);
        static bool isSubStr(string s1, string s2);
};

istream & operator >>(istream &in, Table &MyTab);
ostream & operator <<(ostream &out, Table &MyTab);

class Tab_Sort_Comp {
    public:
        Tab_Sort_Comp(long column, int keytype);
        int operator()(const Delimited_Row &left, const Delimited_Row &right);
    private:
        long SortCol;
        int SortKey;
};

#endif

/*
        textfiles.h:
                Description: A replacement to my old row and table classes which uses the C++ STL.
                Contents:
                        Delimited_Row (class)
                                Tab_Delimited_Row (class)
                                Comma_Delimited_Row (class)
                        Table (class)

                -------------------------------------------------
                Delimited_Row:
                        derived from vector<string>
                        virtual members:
                                set_delimiter(char *)
                                char *get_delimiter()
                                Delimited_Row & operator =
                                string & operator string()
                                string & cell(long column=0)
                                istream& operator >>
                                ostream & operator <<
                        private members:
                                parse_in(string)
                                string parse_out();
                                string delimiter;
                                vector<string> row;
                -------------------------------------------------
                Tab_Delimited_Row:
                        derived from Delimited_Row
                        constructor calls set_delimiter("\t");
                -------------------------------------------------
                Comma_Delimited_Row
                        derived from Delimited_Row
                        constructor calls set_delimiter(",");
                -------------------------------------------------
                Table:
                        private vector<Delimited_Row>
                        virtual members:
                                case sensitive searching:
                                        find(char *what, long row=0, long column=0) 
					    includes column in search, starts at row
                                        scan(char *what, long row=0, long column=0)
					    includes column in search, starts at row, matches substring
                                case insensitive searching:
                                        ifind(char *what, long row=0, long column=0)
					    includes column in search, starts at row
                                        iscan(char *what, long row=0, long column=0)
					    includes column in search, starts at row, matches substring
				Sort(cell,keytype)
				    Key Types:
				        0,1 = String/Ascending
				        -1 = String/Descending
				        2 = Integer/Ascending
				        -2 = Integer/Descending
				        3 = Real/Ascending
				        -3 = Real/Descending
				Delimited_Row & getrow(long row=0)
				    returns reference suitable for use as lvalue
                                string & cell(long row=0, long column=0)
				    returns reference suitable for use as lvalue
                                istream & operator >>
                                ostream & operator <<
                -------------------------------------------------
*/
